use AdventureWorks2008R2
go

--	EXCEPT 
--	1a. Sales Person that only sold in territory 1
select	soh.SalesPersonID
from	sales.SalesOrderHeader as soh
where	soh.TerritoryID = 1
and		soh.SalesPersonID is not null
EXCEPT 
select	soh.SalesPersonID
from	sales.SalesOrderHeader as soh
where	soh.TerritoryID <> 1
and		soh.SalesPersonID is not null

--	1b. Verify where sales person 280 sold
select	SalesPersonID
		, soh.TerritoryID
		, COUNT(*) Territory_Count
from	sales.SalesOrderHeader as soh
where	soh.SalesPersonID = 280
group by soh.SalesPersonID, soh.TerritoryID

--	2a. The sliding scale 
--	Create a temp table that will similuate our partitioned table
IF OBJECT_ID('tempdb..#SalesPartition') IS NOT NULL DROP TABLE #SalesPartition
create table #SalesPartition (
	  SalesOrderID	int
	, OrderDate		datetime
	, SalesPersonID	int
)

--	The first load is January 1 - May 1
insert into #SalesPartition(SalesOrderID, OrderDate, SalesPersonID)
select	SalesOrderID, OrderDate, SalesPersonID
from	sales.SalesOrderHeader as soh
where	soh.OrderDate >= '2014-01-01'
and		soh.OrderDate < '2014-05-15'

--	Some data beyond May 1st got through!

--	How many values from May 2014 are already in the table?
--	Test with Intersect! 
select	soh.SalesOrderID
from	sales.SalesOrderHeader as soh
where	soh.OrderDate >= '2014-05-01'
and		soh.OrderDate < '2014-06-01'
INTERSECT
select	sp.SalesOrderID
from	#SalesPartition as sp

--	Display a list of data from May 2015
--	1,210 records are already in the table
select    sp.OrderDate
		, COUNT(*) as DateCount
from	#SalesPartition as sp
where	sp.OrderDate >= '2014-05-01'
group by sp.OrderDate
with rollup
order by sp.OrderDate

--	Ensure that this data does not get reinserted into the partitioned table
--	Use except to exclude any data that is already in the table
insert into #SalesPartition(SalesOrderID, OrderDate, SalesPersonID)
select	SalesOrderID, OrderDate, SalesPersonID
from	sales.SalesOrderHeader as soh
where	soh.SalesOrderID in (
			select	soh.SalesOrderID
			from	sales.SalesOrderHeader as soh
			where	soh.OrderDate >= '2014-05-01'
			and		soh.OrderDate < '2014-06-01'
			except
			select	sp.SalesOrderID
			from	#SalesPartition as sp
		)
		
--	Display the final list of data from May 2015
select    sp.OrderDate
		, COUNT(*) as DateCount
from	#SalesPartition as sp
where	sp.OrderDate >= '2014-05-01'
group by sp.OrderDate
with rollup
order by sp.OrderDate

--	Verify with the source
select	count(soh.SalesOrderID) as May_Total
from	sales.SalesOrderHeader as soh
where	soh.OrderDate >= '2014-05-01'
and		soh.OrderDate < '2014-06-01'